var ValidacoesEstoque_Service = function(ValidacoesEstoque_Repository, ScreenService, metaDataFactory) {

    var TIPO_OPERACAO = {
        'ENTRADA':    'E',
        'SAIDA':      'S',
        'AJUSTE_IMP': 'A'
    };

    var TIPO_SISTEMA = [
        'DFE', 'DFS', 'EST', 'FAI', 'FAZ', 'FAC', 'REC', 'PAG'
    ];

    var TIPO_CUSTO_PADRAO = '03',
        VAZIO_EST         = ' ';

    this.validaDataLancamento = function(filial, dataLancamento, operacao, sistema, dataEmissao) {
        var validaOperacaoSistema = (!TIPO_OPERACAO.hasOwnProperty(operacao)) || TIPO_SISTEMA.indexOf(sistema) === -1;

        if(!dataLancamento || !operacao || validaOperacaoSistema) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        operacao = TIPO_OPERACAO[operacao];
        dataLancamento = moment(dataLancamento, "DD/MM/YYYY").format('DD/MM/YYYY');
        dataEmissao = !dataEmissao ? dataLancamento : moment(dataEmissao, "DD/MM/YYYY").format('DD/MM/YYYY');

        return ValidacoesEstoque_Repository.validaDataLancamento(filial, dataLancamento, operacao, sistema, dataEmissao);
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Possui como comportamento padrão a
     * atualização de todos os itens de um lançamento de
     * uma unidade, com verificação de irregularidades e
     * abortamento das operações no caso de invalidez
     *
     * Permite porém que a verificação de irregularidades
     * não seja efetuada ou que as operações não sejam
     * abortadas após a verificação através da passagem
     * dos parâmetros "verifyIrregularities" e
     * "abortAtIrregularities". Note que o último não
     * pode ser verdadeiro se o primeiro for falso.
     *
     * Permite também que sejam selecionados um ou mais
     * itens para a execução da atualização de estoque
     * através da passagem de seus códigos na variável
     * "seqItemCodes". Ela é nula no comportamento
     * padrão, o que garante que todos os itens do
     * lançamento sejam atualizados.
     *
     * @param string  cdfilial
     * @param string  nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array   seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function(cdfilial, nrlancestq, boolConsolidateToStock, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //ensures that "seqItemCodes" will never be "undefined", as such value doesn't exist in PHP
        seqItemCodes = seqItemCodes || null;

        //sets default flag values if and only if they haven't been passed
        if(typeof verifyIrregularities === 'undefined')
            verifyIrregularities = true;
        if(typeof abortAtIrregularities === 'undefined')
            abortAtIrregularities = verifyIrregularities;
		if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;

        //checks for invalid parameter values and returns rejected promise
        if(!cdfilial || !nrlancestq ||
           (typeof verifyIrregularities !== 'boolean') ||
           (typeof abortAtIrregularities !== 'boolean') ||
		   (typeof boolConsolidateToStock !== 'boolean') ||
           (!verifyIrregularities && abortAtIrregularities)) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';

            return Promise.reject(response);
        }

        //calls repository's request function, returning its promise
        return ValidacoesEstoque_Repository.atualizaEstoque(
            cdfilial, nrlancestq, verifyIrregularities,
            abortAtIrregularities, seqItemCodes, boolConsolidateToStock
        );
    };

    this.zeraInventario = function(filial, nrLancamento, idordemclc, boolConsolidateToStock) {
        if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;
            
        idordemclc = (idordemclc !== 'U')? 0 :'U';

        if(!filial || !nrLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        return ValidacoesEstoque_Repository.zeraInventario(filial, nrLancamento, idordemclc, boolConsolidateToStock);
    };

    this.consultaCustoQuantidadeProduto = function(filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote) {
        if(!filial || !produto || !dtLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        almoxarifado = almoxarifado || VAZIO_EST;
        localEstoque = localEstoque || VAZIO_EST;

        lote    =    lote || VAZIO_EST;
        subLote = subLote || VAZIO_EST;

        tipoCusto = tipoCusto || TIPO_CUSTO_PADRAO;

        return ValidacoesEstoque_Repository.consultaCustoQuantidadeProduto(
            filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote
        );
    };
    
    this.openPosicaoDeEstoque = function(widget) {
    	var popup = this.getPosicaoDeEstoquePopup(widget);

		ScreenService.openPopup(popup);

		popup.currentRow = widget.currentRow;
		popup.parentWidget = widget;

		this.getDataForPosicaoDeEstoquePopup(popup);
    };
    
    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	this.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		popup.parentWidget.currentRow.QTESTOQDIA = posestdia.QTESTOQDIA;
    		}
    		else{
    			popup.currentRow = 0;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };
    
    this.getPosicaoDeEstoquePopup = function(parentWidget) {
        var widget = parentWidget.posestPopup;

        if(!widget) {
            widget = this.getPosicaoDeEstoquePopUpObject();
            widget.container = parentWidget.container;
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

            widget = metaDataFactory.widgetFactory(widget, parentWidget);

            parentWidget.posestPopup = widget;
            widget.parent = parentWidget;
        } else
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

        return widget;
    };

    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ValidacoesEstoque_Service.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "VRCUSTOPROD",
                    "label": "Custo",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": true,
                                    "precision": 3,
                                    "thousands": ",",
                                    "decimal": "."
                            }
                    },
                    "events": []
                    
                },
                
            ]
        };
    };
    
    this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ 
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

	    return ValidacoesEstoque_Repository.getPosicaoDeEstoquePorProduto(filter).then(function(success){
	    	return success.dataset.posestdia[0];
	    });
	};
};

Configuration (function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Service', ValidacoesEstoque_Service);
});